package auth

import (
	"encoding/base64"
	"fmt"
	"github.com/pquerna/otp"
	"github.com/pquerna/otp/totp"
	"gitlab.bob.co.za/bob-public-utils/bobgroup-go-utils/encryption"
	"gitlab.bob.co.za/bob-public-utils/bobgroup-go-utils/errors"
)

type TwoFactorAuthSetup struct {
	TwoFactorAuthPasscode *string `json:"two_factor_auth_passcode"`
}

type TwoFactorAuthResponse struct {
	AuthURL string `json:"auth_url,omitempty"`
}

func Validate2FA(twoFactorAuthPasscode, encryptedTwoFactorAuthKey, encryptionKeySecret string, isDebug bool) error {
	if twoFactorAuthPasscode == "" {
		return errors.Error("Two-factor authentication passcode is required")
	}

	// Decrypt the key
	encryptionKey, err := encryption.GetJWTEncryptionKey(encryptionKeySecret, isDebug)
	if err != nil {
		return errors.Error("Decryption failed")
	}

	decodedString, err := base64.StdEncoding.DecodeString(encryptedTwoFactorAuthKey)
	if err != nil {
		return err
	}

	decryptedKey, err := encryption.Decrypt(string(decodedString), fmt.Sprintf("%*s", 32, encryptionKey))
	if err != nil {
		return errors.Error("Two-factor decryption failed")
	}

	// Validate the passcode against the key
	isValid := totp.Validate(twoFactorAuthPasscode, decryptedKey)
	if !isValid {
		return errors.Error("Two-factor authentication passcode is invalid")
	}

	return nil
}

func Generate2FAKey(userEmail, siteDomain string) (*otp.Key, error) {
	secret, err := totp.Generate(totp.GenerateOpts{
		Issuer:      siteDomain,
		AccountName: userEmail,
	})

	if err != nil {
		return nil, err
	}

	return secret, nil
}
